/**
 * SPDX-FileCopyrightText: (c) 2000 Liferay, Inc. https://liferay.com
 * SPDX-License-Identifier: LGPL-2.1-or-later OR LicenseRef-Liferay-DXP-EULA-2.0.0-2023-06
 */

import {Text} from '@clayui/core';
import ClayLoadingIndicator from '@clayui/loading-indicator';
import {FrontendDataSet} from '@liferay/frontend-data-set-web';
import {API, stringUtils} from '@liferay/object-js-components-web';
import {createResourceURL, fetch, sub} from 'frontend-js-web';
import React, {useEffect, useState} from 'react';

import {
	IFDSTableProps,
	defaultDataSetProps,
	fdsItem,
	formatActionURL,
} from '../../utils/fds';
import FDSSourceDataRenderer from '../FDSPropsTransformer/FDSSourceDataRenderer';
import LabelRenderer from '../LabelRenderer';
import ModalObjectFieldDeletionNotAllowed from '../ModalObjectFieldDeletionNotAllowed';
import {ModalAddObjectField} from './ModalAddObjectField';
import {ModalDeleteObjectField} from './ModalDeleteObjectField';
import {handleTriggerDeleteObjectField} from './deleteObjectFieldUtil';

interface ObjectFieldItemData {
	itemData: ObjectField;
}

interface FieldsProps extends IFDSTableProps {
	baseResourceURL: string;
}

export default function Fields({
	apiURL,
	baseResourceURL,
	creationMenu,
	formName,
	id,
	items,
	objectDefinitionExternalReferenceCode,
	style,
	url,
}: FieldsProps) {
	const [creationLanguageId, setCreationLanguageId] =
		useState<Liferay.Language.Locale>();

	const [deletedObjectField, setDeletedObjectField] =
		useState<ObjectField | null>(null);

	const [loadingFDS, setLoadingFDS] = useState<boolean>(false);

	const [objectFieldBusinessTypes, setObjectFieldBusinessTypes] =
		useState<Map<string, ObjectFieldBusinessType>>();

	const [objectFieldDeleteInfo, setObjectFieldDeleteInfo] =
		useState<ObjectFieldDeleteInfoProps>({
			deleteLastPublishedObjectDefinitionObjectField: false,
			deleteObjectFieldObjectValidationRuleSetting: false,
			showObjectFieldDeletionConfirmationModal: false,
			showObjectFieldDeletionNotAllowedModal: false,
		});

	const [showAddFieldModal, setShowAddFieldModal] = useState(false);

	useEffect(() => {
		Liferay.on('addObjectField', () => setShowAddFieldModal(true));

		return () => Liferay.detach('addObjectField');
	}, []);

	useEffect(() => {
		const makeFetch = async () => {
			setLoadingFDS(true);

			const objectDefinitionResponse =
				await API.getObjectDefinitionByExternalReferenceCode(
					objectDefinitionExternalReferenceCode
				);

			const url = createResourceURL(baseResourceURL, {
				objectDefinitionId: objectDefinitionResponse.id,
				p_p_resource_id:
					'/object_definitions/get_object_field_business_types',
			}).href;

			const objectFieldBusinessTypesResponse = await fetch(url, {
				method: 'GET',
			});

			const {objectFieldBusinessTypes: newObjectFieldBusinessTypes} =
				(await objectFieldBusinessTypesResponse.json()) as {
					objectFieldBusinessTypes: ObjectFieldBusinessType[];
				};

			const objectFieldBusinessTypesMap = new Map<
				string,
				ObjectFieldBusinessType
			>(
				newObjectFieldBusinessTypes.map((objectFieldBusinessType) => [
					objectFieldBusinessType.businessType,
					objectFieldBusinessType,
				])
			);

			setCreationLanguageId(objectDefinitionResponse.defaultLanguageId);
			setObjectFieldBusinessTypes(objectFieldBusinessTypesMap);

			setTimeout(() => {
				setLoadingFDS(false);
			}, 200);
		};

		makeFetch();
	}, [baseResourceURL, objectDefinitionExternalReferenceCode]);

	function objectFieldBusinessTypeDataRenderer({
		itemData,
	}: ObjectFieldItemData) {
		return objectFieldBusinessTypes?.get(itemData.businessType)?.label;
	}

	function objectFieldLabelDataRenderer({
		itemData,
		openSidePanel,
		value,
	}: fdsItem<ObjectField>) {
		return (
			<LabelRenderer
				onClick={() => {
					openSidePanel({
						url: formatActionURL(url, itemData.id),
					});
				}}
				value={value}
			/>
		);
	}

	function objectFieldLocalizedDataRenderer({itemData}: ObjectFieldItemData) {
		return itemData.localized
			? Liferay.Language.get('yes')
			: Liferay.Language.get('no');
	}

	function objectFieldMandatoryDataRenderer({itemData}: ObjectFieldItemData) {
		return itemData.required
			? Liferay.Language.get('yes')
			: Liferay.Language.get('no');
	}

	const dataSetProps = {
		...defaultDataSetProps,
		apiURL,
		creationMenu,
		customDataRenderers: {
			FDSSourceDataRenderer,
			objectFieldBusinessTypeDataRenderer,
			objectFieldLabelDataRenderer,
			objectFieldLocalizedDataRenderer,
			objectFieldMandatoryDataRenderer,
		},
		formName,
		id,
		itemsActions: items,
		namespace:
			'_com_liferay_object_web_internal_object_definitions_portlet_ObjectDefinitionsPortlet_',
		onActionDropdownItemClick({
			action,
			itemData,
		}: {
			action: {data: {id: string}};
			itemData: ObjectField;
		}) {
			if (action.data.id === 'deleteObjectField') {
				const makeFetch = async () => {
					handleTriggerDeleteObjectField({
						baseResourceURL,
						objectFieldId: itemData?.id,
						objectFieldLabel: stringUtils.getLocalizableLabel(
							creationLanguageId!,
							itemData.label,
							itemData.name
						),
						onAfterDelete: () => {
							setTimeout(() => window.location.reload(), 1500);
						},
						setObjectFieldDeleteInfo,
					});

					setDeletedObjectField(itemData);
				};

				makeFetch();
			}
		},
		portletId:
			'com_liferay_object_web_internal_object_definitions_portlet_ObjectDefinitionsPortlet',
		style,
		views: [
			{
				contentRenderer: 'table',
				label: 'Table',
				name: 'table',
				schema: {
					fields: [
						{
							contentRenderer: 'objectFieldLabelDataRenderer',
							expand: false,
							fieldName: 'label',
							label: Liferay.Language.get('label'),
							localizeLabel: true,
							sortable: true,
						},
						{
							contentRenderer:
								'objectFieldBusinessTypeDataRenderer',
							expand: false,
							fieldName: 'businessType',
							label: Liferay.Language.get('type'),
							localizeLabel: true,
							sortable: false,
						},
						{
							contentRenderer: 'objectFieldMandatoryDataRenderer',
							expand: false,
							fieldName: 'mandatory',
							label: Liferay.Language.get('mandatory'),
							localizeLabel: true,
							sortable: false,
						},
						{
							contentRenderer: 'FDSSourceDataRenderer',
							expand: false,
							fieldName: 'source',
							label: Liferay.Language.get('source'),
							localizeLabel: true,
							sortable: false,
						},
						{
							contentRenderer: 'objectFieldLocalizedDataRenderer',
							expand: false,
							fieldName: 'localized',
							label: Liferay.Language.get('translatable'),
							localizeLabel: true,
							sortable: false,
						},
					],
				},
				thumbnail: 'table',
			},
		],
	};

	return (
		<>
			{loadingFDS ? (
				<ClayLoadingIndicator />
			) : (
				<FrontendDataSet {...dataSetProps} />
			)}

			{showAddFieldModal && (
				<ModalAddObjectField
					baseResourceURL={baseResourceURL}
					creationLanguageId={
						creationLanguageId as Liferay.Language.Locale
					}
					objectDefinitionExternalReferenceCode={
						objectDefinitionExternalReferenceCode
					}
					onAfterSubmit={() => {
						setShowAddFieldModal(false);
						window.location.reload();
					}}
					setVisibility={setShowAddFieldModal}
				/>
			)}

			{objectFieldDeleteInfo.showObjectFieldDeletionConfirmationModal && (
				<ModalDeleteObjectField
					handleOnClose={() =>
						setObjectFieldDeleteInfo(
							(prevState: ObjectFieldDeleteInfoProps) => ({
								...prevState,
								showObjectFieldDeletionConfirmationModal: false,
							})
						)
					}
					objectField={deletedObjectField as ObjectField}
					onAfterSubmit={() => {
						setTimeout(() => window.location.reload(), 1500);
					}}
					setObjectField={setDeletedObjectField}
				/>
			)}

			{!!deletedObjectField &&
				objectFieldDeleteInfo.showObjectFieldDeletionNotAllowedModal && (
					<ModalObjectFieldDeletionNotAllowed
						content={
							objectFieldDeleteInfo.deleteObjectFieldObjectValidationRuleSetting ? (
								<Text>
									{sub(
										Liferay.Language.get(
											'the-object-field-x-cannot-be-deleted-because-it-is-the-only-custom-object-field-of-the-published-object-definition'
										),
										`${stringUtils.getLocalizableLabel(
											creationLanguageId as Liferay.Language.Locale,
											deletedObjectField.label,
											deletedObjectField.name
										)}`
									)}
								</Text>
							) : (
								<Text>
									{sub(
										Liferay.Language.get(
											'the-object-field-x-cannot-be-deleted-because-it-is-used-in-a-unique-composite-key-validation'
										),
										`${stringUtils.getLocalizableLabel(
											creationLanguageId as Liferay.Language.Locale,
											deletedObjectField.label,
											deletedObjectField.name
										)}`
									)}
								</Text>
							)
						}
						onVisibilityChange={() =>
							setObjectFieldDeleteInfo({
								...objectFieldDeleteInfo,
								showObjectFieldDeletionNotAllowedModal: false,
							})
						}
					/>
				)}
		</>
	);
}
