/**
 * SPDX-FileCopyrightText: (c) 2000 Liferay, Inc. https://liferay.com
 * SPDX-License-Identifier: LGPL-2.1-or-later OR LicenseRef-Liferay-DXP-EULA-2.0.0-2023-06
 */

import ClayAlert from '@clayui/alert';
import {
	API,
	Card,
	Input,
	SidePanelForm,
	SingleSelect,
	openToast,
	saveAndReload,
} from '@liferay/object-js-components-web';
import {InputLocalized} from 'frontend-js-components-web';
import React from 'react';

import {
	ObjectRelationshipFormBase,
	useObjectRelationshipForm,
} from './ObjectRelationshipFormBase';
import SelectObjectRelationship from './SelectObjectRelationship';

interface EditObjectRelationshipProps {
	baseResourceURL: string;
	hasUpdateObjectDefinitionPermission: boolean;
	objectDefinitionExternalReferenceCode: string;
	objectRelationship: ObjectRelationship;
	objectRelationshipDeletionTypes: LabelValueObject[];
	parameterRequired: boolean;
	restContextPath: string;
}

export default function EditObjectRelationship({
	baseResourceURL,
	hasUpdateObjectDefinitionPermission,
	objectDefinitionExternalReferenceCode,
	objectRelationship: initialValues,
	objectRelationshipDeletionTypes,
	parameterRequired,
	restContextPath,
}: EditObjectRelationshipProps) {
	const onSubmit = async (objectRelationship: ObjectRelationship) => {
		try {
			if (!Liferay.FeatureFlags['LPS-187142']) {
				delete objectRelationship.edge;
			}

			await API.putObjectRelationship(objectRelationship);
			saveAndReload();

			openToast({
				message: Liferay.Language.get(
					'the-object-relationship-was-updated-successfully'
				),
			});
		}
		catch (error: unknown) {
			const {message} = error as Error;

			openToast({message, type: 'danger'});
		}
	};

	const {errors, handleChange, handleSubmit, setValues, values} =
		useObjectRelationshipForm({
			initialValues,
			onSubmit,
			parameterRequired,
		});

	const readOnly =
		!hasUpdateObjectDefinitionPermission ||
		values.reverse ||
		initialValues.system;

	return (
		<SidePanelForm
			customLabel={{
				displayType: values.reverse ? 'info' : 'success',
				message: values.reverse
					? Liferay.Language.get('child')
					: Liferay.Language.get('parent'),
			}}
			onSubmit={handleSubmit}
			readOnly={readOnly}
			title={Liferay.Language.get('relationship')}
		>
			<Card title={Liferay.Language.get('basic-info')}>
				{values.reverse && (
					<ClayAlert
						displayType="warning"
						title={`${Liferay.Language.get('warning')}:`}
					>
						{Liferay.Language.get(
							'reverse-object-relationships-cannot-be-updated'
						)}
					</ClayAlert>
				)}

				<InputLocalized
					disabled={readOnly}
					error={errors.label}
					label={Liferay.Language.get('label')}
					onChange={(label) => setValues({label})}
					required
					translations={values.label as LocalizedValue<string>}
				/>

				<ObjectRelationshipFormBase
					baseResourceURL={baseResourceURL}
					errors={errors}
					handleChange={handleChange}
					objectDefinitionExternalReferenceCode1={
						objectDefinitionExternalReferenceCode
					}
					readonly
					setValues={setValues}
					values={values}
				/>

				<SingleSelect
					disabled={
						readOnly ||
						(Liferay.FeatureFlags['LPS-187142'] && values.edge)
					}
					items={objectRelationshipDeletionTypes}
					label={Liferay.Language.get('deletion-type')}
					onSelectionChange={(value) =>
						setValues({deletionType: value as string})
					}
					required
					selectedKey={values.deletionType}
				/>
			</Card>

			{parameterRequired && values.type === 'oneToMany' && (
				<Card title={Liferay.Language.get('parameters')}>
					<Input
						label={Liferay.Language.get('api-endpoint')}
						readOnly
						value={restContextPath}
					/>

					<SelectObjectRelationship
						error={errors.parameterObjectFieldName}
						objectDefinitionExternalReferenceCode1={
							values.objectDefinitionExternalReferenceCode2 as string
						}
						onChange={(parameterObjectFieldName) =>
							setValues({parameterObjectFieldName})
						}
						value={values.parameterObjectFieldName}
					/>
				</Card>
			)}
		</SidePanelForm>
	);
}
